#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

# This file is to be included for configuring application which relates
# to orchestration(Heat) functions.

import hashlib
import json

from django.utils.translation import pgettext_lazy


def calculate_md5(target):
    if not target:
        return ''

    md5 = hashlib.md5()
    for chunk in target.chunks():
        md5.update(chunk)

    return md5.hexdigest()


def convert_raw_input_to_api_format(value):
    if value == '':
        return None

    value = value.replace('\r\n', '\n')
    data = value.split('\n')
    input_dict = {}
    for datum in data:
        if datum == "":
            continue
        k, v = datum.split('=')
        input_dict.update({k: v})
    inp = json.dumps(input_dict)
    return inp


FUNCTION_ENGINE_STATUS_CHOICES = (
    ("Creating", None),
    ("Available", True),
    ("Upgrading", True),
    ("Error", False),
    ("Deleting", None),
    ("Running", None),
    ("Done", True),
    ("Paused", True),
    ("Cancelled", True),
    ("Success", True),
    ("Failed", False),
)


FUNCTION_ENGINE_STATUS_DISPLAY_CHOICES = (
    ("creating", pgettext_lazy("current status of runtime", u"Creating")),
    ("available",
     pgettext_lazy("current status of runtime", u"Available")),
    ("upgrading",
     pgettext_lazy("current status of runtime", u"Upgrading")),
    ("error", pgettext_lazy("current status of runtime", u"Error")),
    ("deleting", pgettext_lazy("current status of runtime", u"Deleting")),
    ("running", pgettext_lazy("current status of runtime", u"Running")),
    ("done", pgettext_lazy("current status of runtime", u"Done")),
    ("paused", pgettext_lazy("current status of runtime", u"Paused")),
    ("cancelled",
     pgettext_lazy("current status of runtime", u"Cancelled")),
    ("success", pgettext_lazy("current status of runtime", u"Success")),
    ("failed", pgettext_lazy("current status of runtime", u"Failed")),
)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import io

from django import shortcuts

from django.http import HttpResponse
from django.urls import reverse
from django.urls import reverse_lazy
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import messages
from horizon import tables
from horizon import tabs

from horizon.utils import memoized

from qinling_dashboard import api

from qinling_dashboard import exceptions as q_exc

from qinling_dashboard.content.executions import views as ex_views

from qinling_dashboard.content.functions import forms as project_forms
from qinling_dashboard.content.functions import tables as project_tables
from qinling_dashboard.content.functions import tabs as project_tabs


class CreateFunctionExecutionView(ex_views.CreateExecutionView):

    modal_header = submit_label = page_title = _("Create Execution")
    submit_url = "horizon:project:functions:create_execution"
    success_url = "horizon:project:functions:detail"

    def get_context_data(self, **kwargs):
        context = super(CreateFunctionExecutionView, self).\
            get_context_data(**kwargs)
        submit_url = reverse(self.submit_url,
                             args=(self.kwargs['function_id'],))
        context['submit_url'] = submit_url
        return context

    def get_success_url(self):
        function_id = self.kwargs['function_id']
        result = reverse(self.success_url, args=(function_id,))
        result += "?tab=function_details__executions_of_this_function"
        return result


class CreateFunctionView(forms.ModalFormView):

    form_class = project_forms.CreateFunctionForm
    modal_header = submit_label = page_title = _("Create Function")
    template_name = "project/functions/create_function.html"
    submit_url = reverse_lazy("horizon:project:functions:create")
    success_url = reverse_lazy("horizon:project:functions:index")


class UpdateFunctionView(forms.ModalFormView):

    form_class = project_forms.UpdateFunctionForm
    modal_header = submit_label = page_title = _("Update Function")
    template_name = "project/functions/update_function.html"
    submit_url = "horizon:project:functions:update"
    success_url = reverse_lazy("horizon:project:functions:index")

    @memoized.memoized_method
    def get_object(self, *args, **kwargs):
        function_id = self.kwargs['function_id']
        try:
            return api.qinling.function_get(self.request, function_id)
        except Exception:
            redirect = reverse("horizon:project:functions:index")
            msg = _('Unable to retrieve function details.')
            exceptions.handle(self.request, msg, redirect=redirect)

    def get_context_data(self, **kwargs):
        context = super(UpdateFunctionView, self).\
            get_context_data(**kwargs)
        function_id = self.kwargs['function_id']
        context['function_id'] = function_id
        context['submit_url'] = reverse(self.submit_url, args=(function_id,))
        return context

    def get_initial(self):
        initial = super(UpdateFunctionView, self).get_initial()
        func = self.get_object()
        code = getattr(func, 'code', {})
        code_swift = code.get('swift', {})
        source = code.get('source', '')

        initial.update({
            'function_id': func.id,
            'name': getattr(func, 'name', ''),
            'description': getattr(func, 'description', ''),
            'cpu': getattr(func, 'cpu', ''),
            'memory_size': getattr(func, 'memory_size', ''),
            'runtime_id': getattr(func, 'runtime_id', ''),
            'entry': getattr(func, 'entry', ''),
            'code_type': code.get('source', ''),
            'swift_container': code_swift.get('container', ''),
            'swift_object': code_swift.get('object', ''),
            'image': code.get('image', ''),
            'source': source,
        })
        return initial


class CreateFunctionVersionView(forms.ModalFormView):

    form_class = project_forms.CreateFunctionVersionForm
    modal_header = submit_label = page_title = _("Create Version")
    template_name = "project/functions/create_version.html"
    submit_url = "horizon:project:functions:create_version"
    success_url = "horizon:project:functions:detail"

    def get_success_url(self):
        function_id = self.kwargs['function_id']
        result = reverse(self.success_url, args=(function_id,))
        result += "?tab=function_details__versions_of_this_function"
        return result

    def get_context_data(self, **kwargs):
        context = super(CreateFunctionVersionView, self).\
            get_context_data(**kwargs)
        function_id = self.kwargs['function_id']
        context['function_id'] = function_id
        context['submit_url'] = reverse(self.submit_url, args=(function_id,))
        return context

    def get_initial(self):
        initial = super(CreateFunctionVersionView, self).get_initial()
        initial.update({
            'function_id': self.kwargs["function_id"],
        })
        return initial


class IndexView(tables.DataTableView):

    table_class = project_tables.FunctionsTable
    page_title = _("Functions")

    def get_data(self):
        try:
            functions = api.qinling.functions_list(self.request)
        except Exception:
            functions = []
            msg = _('Unable to retrieve functions list.')
            exceptions.handle(self.request, msg)
        return functions


class DetailView(tabs.TabbedTableView):
    tab_group_class = project_tabs.FunctionDetailTabs
    template_name = 'project/functions/detail.html'
    page_title = _("Function Details: {{ resource_name }}")
    failure_url = reverse_lazy('horizon:project:functions:index')

    @memoized.memoized_method
    def _get_data(self):
        function_id = self.kwargs['function_id']
        try:
            function = api.qinling.function_get(self.request, function_id)
        except Exception:
            exceptions.handle(
                self.request,
                _('Unable to retrieve '
                  'details for Function "%s".') % function_id,
                redirect=self.failure_url
            )
        return function

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        func = self._get_data()
        table = project_tables.FunctionsTable(self.request)
        resource_name = func.name if func.name else func.id
        context["function"] = func
        context["url"] = self.get_redirect_url()
        context["actions"] = table.render_row_actions(func)
        context["table_id"] = "runtime"
        context["resource_name"] = resource_name
        context["object_id"] = func.id
        return context

    def _get_executions(self):
        function_id = self.kwargs['function_id']
        try:
            executions = api.qinling.executions_list(self.request, function_id)
        except Exception:
            executions = []
            messages.error(self.request, _(
                'Unable to get executions of '
                'this function "%s".') % function_id)
        return executions

    def _get_versions(self):
        function_id = self.kwargs['function_id']
        try:
            versions = api.qinling.versions_list(self.request, function_id)
        except Exception:
            versions = []
            messages.error(self.request, _(
                'Unable to get versions of this function "%s".') % function_id)
        return versions

    def get_tabs(self, request, *args, **kwargs):
        func = self._get_data()
        executions = self._get_executions()
        versions = self._get_versions()
        return self.tab_group_class(request,
                                    function=func,
                                    executions=executions,
                                    versions=versions,
                                    **kwargs)

    @staticmethod
    def get_redirect_url():
        return reverse('horizon:project:functions:index')


class VersionDetailView(tabs.TabView):
    tab_group_class = project_tabs.FunctionVersionDetailTabs
    template_name = 'project/functions/detail_version.html'
    page_title = _("Function Version Details: "
                   "{{ version.function_id }} "
                   "(Version Number={{ version_number }})")
    failure_url = reverse_lazy('horizon:project:functions:index')

    def get_redirect_url(self, version_number=None):
        function_id = self.kwargs['function_id']

        if not version_number:
            return reverse('horizon:project:functions:detail',
                           args=(function_id,))

        return reverse('horizon:project:functions:version_detail',
                       args=(function_id, version_number))

    @memoized.memoized_method
    def get_data(self):
        function_id = self.kwargs['function_id']
        version_number = int(self.kwargs['version_number'])
        try:
            version = api.qinling.version_get(self.request,
                                              function_id,
                                              version_number)
        except Exception:
            exceptions.handle(
                self.request,
                _('Unable to retrieve details for '
                  'function version "%s".') % (function_id, version_number),
                redirect=self.failure_url
            )
        return version

    def get_context_data(self, **kwargs):
        context = super(VersionDetailView, self).get_context_data(**kwargs)
        version = self.get_data()

        table = project_tables.FunctionVersionsTable(self.request)
        context["version"] = version
        context["url"] = self.get_redirect_url()
        context["actions"] = table.render_row_actions(version)
        context["table_id"] = "function_version"
        context["object_id"] = version.version_number
        return context

    def get_tabs(self, request, *args, **kwargs):
        version = self.get_data()
        return self.tab_group_class(request,
                                    version=version,
                                    **kwargs)


def download_function(request, function_id):
    try:
        data = api.qinling.function_download(request, function_id)

        output = io.BytesIO()
        for chunk in data:
            output.write(chunk)
        ctx = output.getvalue()

        response = HttpResponse(ctx, content_type='application/octet-stream')
        response['Content-Length'] = str(len(response.content))

        response['Content-Disposition'] = \
            'attachment; filename=qinling-function-' + function_id + '.zip'
        return response
    except q_exc.NOT_FOUND as ne:
        messages.error(request,
                       _('Error because file not found function: %s') % ne)
    except Exception as e:
        messages.error(request, _('Error downloading function: %s') % e)
        return shortcuts.redirect(request.build_absolute_uri())



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from qinling_dashboard import api
from qinling_dashboard.test import helpers as test


class QinlingApiTests(test.APIMockTestCase):

    # Runtimes
    @mock.patch.object(api.qinling, 'qinlingclient')
    def test_runtimes_list(self, mock_qinlingclient):
        """Test for api.qinling.runtimes_list()"""
        runtimes = self.runtimes.list()

        qclient = mock_qinlingclient.return_value
        qclient.runtimes.list.return_value = runtimes

        result = api.qinling.runtimes_list(self.request)
        self.assertItemsEqual(result, runtimes)

        qclient.runtimes.list.assert_called_once_with()

    @mock.patch.object(api.qinling, 'qinlingclient')
    def test_runtime_get(self, mock_qinlingclient):
        """Test for api.qinling.runtime_get()"""
        runtime = self.runtimes.first()

        qclient = mock_qinlingclient.return_value
        qclient.runtimes.get.return_value = runtime

        result = api.qinling.runtime_get(self.request, runtime.id)
        self.assertEqual(result, runtime)

        qclient.runtimes.get.assert_called_once_with(runtime.id)

    # Functions
    @mock.patch.object(api.qinling, 'qinlingclient')
    def test_function_create(self, mock_qinlingclient):
        """Test for api.qinling.function_create()"""
        func = self.functions.first()

        qclient = mock_qinlingclient.return_value
        qclient.functions.create.return_value = func

        params = {}
        result = api.qinling.function_create(self.request, **params)
        self.assertEqual(result, func)

        qclient.functions.create.assert_called_once_with(**params)

    @mock.patch.object(api.qinling, 'qinlingclient')
    def test_function_update(self, mock_qinlingclient):
        """Test for api.qinling.function_update()"""
        func = self.functions.first()

        qclient = mock_qinlingclient.return_value
        qclient.functions.update.return_value = func

        params = {}
        result = api.qinling.function_update(self.request, func.id, **params)
        self.assertEqual(result, func)

        qclient.functions.update.assert_called_once_with(func.id, **params)

    @mock.patch.object(api.qinling, 'qinlingclient')
    def test_function_delete(self, mock_qinlingclient):
        """Test for api.qinling.function_delete()"""
        func = self.functions.first()

        qclient = mock_qinlingclient.return_value
        qclient.functions.delete.return_value = None

        result = api.qinling.function_delete(self.request, func.id)
        self.assertIsNone(result)

        qclient.functions.delete.assert_called_once_with(func.id)

    @mock.patch.object(api.qinling, 'qinlingclient')
    def test_functions_list(self, mock_qinlingclient):
        """Test for api.qinling.functions_list()"""
        functions = self.functions.list()

        qclient = mock_qinlingclient.return_value
        qclient.functions.list.return_value = functions

        result = api.qinling.functions_list(self.request)
        self.assertItemsEqual(result, functions)

        qclient.functions.list.assert_called_once_with()

    @mock.patch.object(api.qinling, 'qinlingclient')
    def test_function_get(self, mock_qinlingclient):
        """Test for api.qinling.function_get()"""
        func = self.functions.first()

        qclient = mock_qinlingclient.return_value
        qclient.functions.get.return_value = func

        result = api.qinling.function_get(self.request, func.id)
        self.assertEqual(result, func)

        qclient.functions.get.assert_called_once_with(func.id)

    @mock.patch.object(api.qinling, 'qinlingclient')
    def test_function_download(self, mock_qinlingclient):
        """Test for api.qinling.function_download()"""
        func = self.functions.first()

        qclient = mock_qinlingclient.return_value
        qclient.functions.get.return_value = func

        result = api.qinling.function_download(self.request, func.id)
        self.assertEqual(result, func)

        qclient.functions.get.assert_called_once_with(func.id, download=True)

    # Function Executions
    @mock.patch.object(api.qinling, 'qinlingclient')
    def test_execution_create(self, mock_qinlingclient):
        """Test for api.qinling.execution_create()"""
        func = self.functions.first()
        execution = self.executions.first()

        qclient = mock_qinlingclient.return_value
        qclient.function_executions.create.return_value = execution

        result = api.qinling.execution_create(self.request, func.id,
                                              version=1, sync=True,
                                              input=None)
        self.assertEqual(result, execution)

        qclient.function_executions.create.assert_called_once_with(func.id,
                                                                   1,
                                                                   True,
                                                                   None)

    @mock.patch.object(api.qinling, 'qinlingclient')
    def test_execution_delete(self, mock_qinlingclient):
        """Test for api.qinling.execution_delete()"""
        execution = self.executions.first()

        qclient = mock_qinlingclient.return_value
        qclient.function_executions.create.return_value = execution

        result = api.qinling.execution_delete(self.request, execution.id)
        self.assertIsNone(result)

        qclient.function_executions.delete.\
            assert_called_once_with(execution.id)

    @mock.patch.object(api.qinling, 'qinlingclient')
    def test_execution_log_get(self, mock_qinlingclient):
        """Test for api.qinling.execution_log_get()"""
        execution = self.executions.first()
        log_contents = "this is log line."

        class FakeResponse(object):
            _content = log_contents

        qclient = mock_qinlingclient.return_value
        qclient.function_executions.http_client.json_request.\
            return_value = FakeResponse(), "dummybody"

        result = api.qinling.execution_log_get(self.request, execution.id)
        self.assertEqual(result, log_contents)

        url = '/v1/executions/%s/log' % execution.id
        qclient.function_executions.http_client.\
            json_request.assert_called_once_with(url, 'GET')

    @mock.patch.object(api.qinling, 'qinlingclient')
    def test_executions_list(self, mock_qinlingclient):
        """Test for api.qinling.executions_list()"""
        executions = self.executions.list()

        qclient = mock_qinlingclient.return_value
        qclient.function_executions.list.return_value = executions

        result = api.qinling.executions_list(self.request)
        self.assertItemsEqual(result, executions)

        qclient.function_executions.list.assert_called_once_with()

    @mock.patch.object(api.qinling, 'qinlingclient')
    def test_execution_get(self, mock_qinlingclient):
        """Test for api.qinling.execution_get()"""
        execution = self.executions.first()

        qclient = mock_qinlingclient.return_value
        qclient.function_executions.get.return_value = execution

        result = api.qinling.execution_get(self.request, execution.id)
        self.assertEqual(result, execution)

        qclient.function_executions.get.assert_called_once_with(execution.id)

    # Function Versions
    @mock.patch.object(api.qinling, 'qinlingclient')
    def test_versions_list(self, mock_qinlingclient):
        """Test for api.qinling.versions_list()"""
        versions = self.versions.list()
        func = self.functions.first()

        this_function_id = func.id

        my_versions = [v for v in versions
                       if v.function_id == this_function_id]

        qclient = mock_qinlingclient.return_value
        qclient.function_versions.list.return_value = my_versions

        result = api.qinling.versions_list(self.request,
                                           this_function_id)
        self.assertItemsEqual(result, my_versions)

        qclient.function_versions.list.\
            assert_called_once_with(this_function_id)

    @mock.patch.object(api.qinling, 'qinlingclient')
    def test_version_get(self, mock_qinlingclient):
        """Test for api.qinling.version_get()"""
        versions = self.versions.list()
        func = self.functions.first()

        this_function_id = func.id
        this_version_number = 1

        my_version = [v for v in versions
                      if v.function_id == this_function_id and
                      v.version_number == this_version_number][0]

        qclient = mock_qinlingclient.return_value
        qclient.function_versions.get.return_value = my_version

        result = api.qinling.version_get(self.request,
                                         this_function_id,
                                         this_version_number)
        self.assertEqual(result, my_version)

        qclient.function_versions.get.\
            assert_called_once_with(this_function_id, this_version_number)

    @mock.patch.object(api.qinling, 'qinlingclient')
    def test_version_create(self, mock_qinlingclient):
        """Test for api.qinling.version_create()"""
        version = self.versions.first()
        func = self.functions.first()

        this_function_id = func.id

        qclient = mock_qinlingclient.return_value
        qclient.function_versions.create.return_value = version

        result = api.qinling.version_create(self.request,
                                            this_function_id,
                                            "description sample")
        self.assertEqual(result, version)

        qclient.function_versions.create.\
            assert_called_once_with(this_function_id, "description sample")



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from qinlingclient.v1 import function
from qinlingclient.v1 import function_execution
from qinlingclient.v1 import function_version
from qinlingclient.v1 import runtime

from qinling_dashboard.test.test_data import utils


PROJECT_ID = "87173d0c07d547bfa1343cabe2e6fe69"
RUNTIME_ID_BASE = "6cb1f505-9a42-4569-a94d-9c2f4b7e7b4{0}"
FUNCTION_ID_BASE = "aacb5032-f8b9-4ad5-8fd4-8017d0d1c54{0}"
EXECUTION_ID_BASE = "f0b5d7f5-d1f1-4f3e-9080-16e4cd918f9{0}"
VERSION_ID_BASE = "30c3e142-2850-4a89-90a7-0a4e4d654f{0}{1}"


def data(TEST):
    TEST.runtimes = utils.TestDataContainer()
    TEST.functions = utils.TestDataContainer()
    TEST.executions = utils.TestDataContainer()
    TEST.versions = utils.TestDataContainer()

    for i in range(10):
        runtime_data = {
            "id": RUNTIME_ID_BASE.format(i),
            "name": "python2.7-runtime-{0}".format(i),
            "status": "available",
            "created_at": "2018-07-11 01:09:13",
            "description": None,
            "image": "openstackqinling/python-runtime",
            "updated_at": "2018-07-11 01:09:28",
            "is_public": True,
            "project_id": PROJECT_ID,
        }
        rt = runtime.Runtime(runtime.RuntimeManager(None), runtime_data)
        TEST.runtimes.add(rt)

    for i in range(10):
        function_data = {
            "id": FUNCTION_ID_BASE.format(i),
            "count": 0,
            "code": "{\"source\": \"package\", "
                    "\"md5sum\": \"976325c9b41bc5a2ddb54f3493751f7e\"}",
            "description": None,
            "created_at": "2018-08-01 08:33:50",
            "updated_at": None,
            "latest_version": 0,
            "memory_size": 33554432,
            "timeout": None,
            "entry": "qinling_test.main",
            "project_id": PROJECT_ID,
            "cpu": 100,
            "runtime_id": RUNTIME_ID_BASE.format(i),
            "name": "github_test"
        }
        func = function.Function(function.FunctionManager(None), function_data)
        TEST.functions.add(func)

    for i in range(10):
        execution_data = {
            "status": "success",
            "project_id": PROJECT_ID,
            "description": None,
            "updated_at": "2018-08-01 06:09:58",
            "created_at": "2018-08-01 06:09:55",
            "sync": True,
            "function_version": 0,
            "result": "{\"duration\": 0.788, \"output\": 30}",
            "input": None,
            "function_id": FUNCTION_ID_BASE.format(i),
            "id": EXECUTION_ID_BASE.format(i),
        }
        execution = function_execution.FunctionExecution(
            function_execution.ExecutionManager(None), execution_data)
        TEST.executions.add(execution)

    # Each mocked function has 10 of version data.
    for i in range(10):
        this_function_id = FUNCTION_ID_BASE.format(i)
        for j in range(10):
            version_data = {
                "count": 0,
                "version_number": j,
                "function_id": this_function_id,
                "description": "",
                "created_at": "2018-08-03 02:01:44",
                "updated_at": None,
                "project_id": PROJECT_ID,
                "id": VERSION_ID_BASE.format(i, j)
            }
            version = function_version.FunctionVersion(
                function_version.FunctionVersionManager(None), version_data)
            TEST.versions.add(version)



# Copyright 2012 Nebula, Inc.
# All rights reserved.

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from django.urls import reverse

from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import messages

from qinling_dashboard import api

# from qinling_dashboard.utils import calculate_md5

from qinling_dashboard import validators


class CreateRuntimeForm(forms.SelfHandlingForm):

    image = forms.CharField(max_length=255,
                            label=_("Image"),
                            validators=[validators.validate_one_line_string],
                            required=True)

    name = forms.CharField(max_length=255,
                           label=_("Name"),
                           validators=[validators.validate_one_line_string],
                           required=False)

    description = forms.CharField(
        max_length=255,
        widget=forms.Textarea(
            attrs={'class': 'modal-body-fixed-width',
                   'rows': 3}),
        label=_("Description"),
        required=False)

    untrusted = forms.BooleanField(
        label=_("Create as Untrusted Image"),
        required=False,
        help_text=_("Check this item if you would like to "
                    "create untrusted runtime"),
        initial=False
    )

    def __init__(self, request, *args, **kwargs):
        super(CreateRuntimeForm, self).__init__(request, *args, **kwargs)

    def handle(self, request, context):
        params = {}

        # basic parameters
        params.update({'image': context.get('image')})

        if context.get('name'):
            params.update({'name': context.get('name')})

        if context.get('description'):
            params.update({'description': context.get('description')})

        if context.get('untrusted'):
            trusted = not bool(context.get('untrusted'))
            params.update({'trusted': trusted})

        try:
            api.qinling.runtime_create(request, **params)
            message = _('Created runtime "%s"') % params.get('name',
                                                             'unknown name')
            messages.success(request, message)
            return True
        except Exception:
            redirect = reverse("horizon:project:runtimes:index")
            exceptions.handle(request,
                              _("Unable to create runtime."),
                              redirect=redirect)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from django.utils.translation import ugettext_lazy as _

from horizon import tables

from qinling_dashboard import api
from qinling_dashboard import utils


class CreateRuntime(tables.LinkAction):
    name = "create"
    verbose_name = _("Create Runtime")
    url = "horizon:project:runtimes:create"
    classes = ("ajax-modal", "btn-create")
    policy_rules = (("function_engine", "runtime:create"),)
    icon = "plus"
    ajax = True


class RuntimesFilterAction(tables.FilterAction):

    def filter(self, table, runtimes, filter_string):
        """Naive case-insensitive search."""
        q = filter_string.lower()
        return [runtime for runtime in runtimes
                if q in runtime.name.lower()]


class UpdateRow(tables.Row):
    ajax = True

    def get_data(self, request, runtime_id):
        execution = api.qinling.runtime_get(request, runtime_id)
        return execution


class RuntimesTable(tables.DataTable):

    id = tables.Column("id",
                       verbose_name=_("Id"),
                       link="horizon:project:runtimes:detail")

    name = tables.Column("name",
                         verbose_name=_("Name"))

    image = tables.Column("image",
                          verbose_name=_("Image"))

    created_at = tables.Column("created_at",
                               verbose_name=_("Created At"))

    updated_at = tables.Column("updated_at",
                               verbose_name=_("Updated At"))

    project_id = tables.Column("project_id",
                               verbose_name=_("Project ID"))

    is_public = tables.Column("is_public",
                              verbose_name=_("Is Public"))

    status = tables.Column(
        "status",
        status=True,
        status_choices=utils.FUNCTION_ENGINE_STATUS_CHOICES,
        display_choices=utils.FUNCTION_ENGINE_STATUS_DISPLAY_CHOICES)

    def __init__(self, request, data=None, needs_form_wrapper=None, **kwargs):
        super(RuntimesTable, self).__init__(request, data,
                                            needs_form_wrapper, **kwargs)
        if not request.user.is_superuser:
            del self.columns["is_public"]

    class Meta(object):
        name = "runtimes"
        verbose_name = _("Runtimes")
        status_columns = ["status"]
        multi_select = True
        row_class = UpdateRow
        table_actions = (CreateRuntime,
                         RuntimesFilterAction,)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

import mock

from django.urls import reverse

from django.utils.http import urlunquote

from qinling_dashboard import api

from qinling_dashboard.test import helpers as test


INDEX_TEMPLATE = 'horizon/common/_data_table_view.html'
INDEX_URL = reverse('horizon:project:executions:index')


class ExecutionsTests(test.TestCase):

    @test.create_mocks({
        api.qinling: [
            'executions_list',
            'execution_delete',
        ]})
    def test_execution_delete(self):
        data_executions = self.executions.list()
        data_execution = self.executions.first()
        execution_id = data_execution.id

        self.mock_executions_list.return_value = data_executions
        self.mock_execution_delete.return_value = None

        form_data = {'action': 'executions__delete__%s' % execution_id}
        res = self.client.post(INDEX_URL, form_data)
        self.assertRedirectsNoFollow(res, INDEX_URL)

        self.mock_execution_delete.assert_called_once_with(
            test.IsHttpRequest(), execution_id)

    @test.create_mocks({
        api.qinling: [
            'execution_create',
            'functions_list',
            'function_get',
            'versions_list',
        ]})
    def test_execution_create(self):
        data_execution = self.executions.first()

        data_functions = self.functions.list()

        data_function = self.functions.first()
        function_id = data_function.id

        data_versions = self.versions.list()
        my_versions = [v for v in data_versions
                       if v.function_id == function_id]

        self.mock_versions_list.return_value = data_versions
        self.mock_function_get.return_value = data_function
        self.mock_functions_list.return_value = data_functions
        self.mock_execution_create.return_value = data_execution
        self.mock_versions_list.return_value = my_versions

        form_data = {'func': function_id,
                     'version': 0,
                     'sync': True,
                     'input_params': 'K1=V1\nK2=V2'}

        url = reverse('horizon:project:executions:create')
        res = self.client.post(url, form_data)

        self.assertNoFormErrors(res)
        self.assertRedirectsNoFollow(res, INDEX_URL)

        input_dict = {'K1': 'V1', 'K2': 'V2'}
        input_params = json.dumps(input_dict)

        self.mock_execution_create.assert_called_once_with(
            test.IsHttpRequest(), function_id, 0, True, input_params)

    @test.create_mocks({
        api.qinling: [
            'execution_create',
            'functions_list',
            'function_get',
            'versions_list',
        ]})
    def test_execution_create_function_input_params_is_not_key_value(self):

        data_functions = self.functions.list()

        data_function = self.functions.first()
        function_id = data_function.id

        data_versions = self.versions.list()
        my_versions = [v for v in data_versions
                       if v.function_id == function_id]

        self.mock_versions_list.return_value = data_versions
        self.mock_function_get.return_value = data_function
        self.mock_functions_list.return_value = data_functions
        self.mock_versions_list.return_value = my_versions

        form_data = {'func': function_id,
                     'version': 0,
                     'sync': True,
                     'input_params': 'K1=V1\nK2='}

        url = reverse('horizon:project:executions:create')
        res = self.client.post(url, form_data)

        expected_msg = "Not key-value pair."
        self.assertContains(res, expected_msg)

    @test.create_mocks({
        api.qinling: [
            'execution_create',
            'functions_list',
            'function_get',
            'versions_list',
        ]})
    def test_execution_create_function_version_does_not_exist(self):

        data_functions = self.functions.list()

        data_function = self.functions.first()
        function_id = data_function.id

        data_versions = self.versions.list()
        my_versions = [v for v in data_versions
                       if v.function_id == function_id]

        self.mock_versions_list.return_value = data_versions
        self.mock_function_get.return_value = data_function
        self.mock_functions_list.return_value = data_functions
        self.mock_versions_list.return_value = my_versions

        invalid_version = 10
        form_data = {'func': function_id,
                     'version': invalid_version,
                     'sync': True,
                     'input_params': ""}

        url = reverse('horizon:project:executions:create')
        res = self.client.post(url, form_data)

        expected_msg = "This function does not " \
                       "have specified version number: %s" % invalid_version
        self.assertContains(res, expected_msg)

    @test.create_mocks({
        api.qinling: [
            'execution_create',
            'functions_list',
            'function_get',
            'versions_list',
        ]})
    def test_execution_create_function_id_is_not_in_choices(self):

        data_functions = self.functions.list()

        data_function = self.functions.first()
        function_id = data_function.id

        data_versions = self.versions.list()
        my_versions = [v for v in data_versions
                       if v.function_id == function_id]

        self.mock_versions_list.return_value = data_versions
        self.mock_function_get.return_value = data_function
        self.mock_functions_list.return_value = data_functions
        self.mock_versions_list.return_value = my_versions

        form_data = {'func': function_id + 'a',  # function does not exist
                     'version': 0,
                     'sync': True,
                     'input_params': ""}

        url = reverse('horizon:project:executions:create')
        res = self.client.post(url, form_data)

        expected_msg = \
            "%s is not one of the available choices." % (function_id + 'a')
        self.assertContains(res, expected_msg)

    # You should not mock api.qinling.executions_list/get itself here,
    # because inside of executions_list, execution.result is converted
    # from str to dict.
    # If you mock everything about this executions_list, above conversion
    # method won't be called then it causes error in table rendering.
    @mock.patch.object(api.qinling, 'qinlingclient')
    def test_index(self, mock_qinlingclient):
        data_executions = self.executions.list()

        qclient = mock_qinlingclient.return_value
        qclient.function_executions.list.return_value = data_executions

        res = self.client.get(INDEX_URL)

        self.assertTemplateUsed(res, INDEX_TEMPLATE)
        executions = res.context['executions_table'].data
        self.assertItemsEqual(executions, self.executions.list())

        qclient.function_executions.list.assert_called_once_with()

    @mock.patch.object(api.qinling, 'qinlingclient')
    def test_index_executions_list_returns_exception(self, mock_qinlingclient):
        # data_executions = self.executions.list()

        qclient = mock_qinlingclient.return_value
        qclient.function_executions.list.side_effect = self.exceptions.qinling

        res = self.client.get(INDEX_URL)

        self.assertTemplateUsed(res, INDEX_TEMPLATE)

        self.assertEqual(len(res.context['executions_table'].data), 0)
        self.assertMessageCount(res, error=1)

        qclient.function_executions.list.assert_called_once_with()

    @mock.patch.object(api.qinling, 'qinlingclient')
    def test_detail(self, mock_qinlingclient):
        execution_id = self.executions.first().id
        data_execution = self.executions.first()

        qclient = mock_qinlingclient.return_value
        qclient.function_executions.get.return_value = data_execution

        url = urlunquote(reverse('horizon:project:executions:detail',
                                 args=[execution_id]))
        res = self.client.get(url)

        result_execution = res.context['execution']

        self.assertEqual(execution_id, result_execution.id)

        self.assertTemplateUsed(res, 'project/executions/detail.html')

        qclient.function_executions.get.assert_called_once_with(execution_id)

    @mock.patch.object(api.qinling, 'qinlingclient')
    def test_detail_execution_get_returns_exception(self, mock_qinlingclient):
        execution_id = self.executions.first().id

        qclient = mock_qinlingclient.return_value
        qclient.function_executions.get.side_effect = self.exceptions.qinling

        url = urlunquote(reverse('horizon:project:executions:detail',
                                 args=[execution_id]))
        res = self.client.get(url)

        redir_url = INDEX_URL
        self.assertRedirectsNoFollow(res, redir_url)

        qclient.function_executions.get.assert_called_once_with(execution_id)

    @test.create_mocks({
        api.qinling: [
            'execution_get',
            'execution_log_get',
        ]})
    def test_detail_execution_log_tab(self):
        execution_id = self.executions.first().id
        log_contents = "this is log line."

        self.mock_execution_get.return_value = self.executions.first()
        self.mock_execution_log_get.return_value = log_contents

        url_base = 'horizon:project:executions:detail_execution_logs'
        url = urlunquote(reverse(url_base, args=[execution_id]))
        res = self.client.get(url)

        result_logs = res.context['execution_logs']

        self.assertTemplateUsed(res, 'project/executions/detail.html')
        self.assertEqual(log_contents, result_logs)

        self.mock_execution_get.assert_has_calls(
            [
                mock.call(test.IsHttpRequest(), execution_id),
            ])
        self.mock_execution_log_get.assert_has_calls(
            [
                mock.call(test.IsHttpRequest(), execution_id),
            ])

    @test.create_mocks({
        api.qinling: [
            'execution_get',
            'execution_log_get',
        ]})
    def test_detail_execution_log_tab_log_get_returns_exception(self):
        execution_id = self.executions.first().id

        self.mock_execution_get.return_value = self.executions.first()
        self.mock_execution_log_get.side_effect = self.exceptions.qinling

        url_base = 'horizon:project:executions:detail_execution_logs'
        url = urlunquote(reverse(url_base, args=[execution_id]))
        res = self.client.get(url)

        result_logs = res.context['execution_logs']

        self.assertTemplateUsed(res, 'project/executions/detail.html')

        self.assertEqual(result_logs, "")

        self.mock_execution_get.assert_has_calls(
            [
                mock.call(test.IsHttpRequest(), execution_id),
            ])
        self.mock_execution_log_get.assert_has_calls(
            [
                mock.call(test.IsHttpRequest(), execution_id),
            ])



# Copyright 2012 Nebula, Inc.
# All rights reserved.

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
Views for managing volumes.
"""

from django.urls import reverse
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import messages

from qinling_dashboard import api
from qinling_dashboard import utils as q_utils
from qinling_dashboard import validators


class CreateExecutionForm(forms.SelfHandlingForm):

    func = forms.ChoiceField(
        label=_("Function"),
        help_text=_("Function to execute."),
        required=True)

    version = forms.IntegerField(
        label=_("Version"),
        required=False,
        initial=0
    )

    sync = forms.BooleanField(
        label=_("Sync"),
        required=False,
        help_text=_("Check this item if you would like sync execution."),
        initial=True
    )

    input_params = forms.CharField(
        label=_('Input'),
        help_text=_('Specify input parmeters like name1=value2. '
                    'One line is equivalent of one input parameter.'),
        validators=[validators.validate_key_value_pairs],
        widget=forms.widgets.Textarea(),
        required=False)

    def __init__(self, request, *args, **kwargs):
        super(CreateExecutionForm, self).__init__(request, *args, **kwargs)
        self.fields['func'].choices = self.get_func_choices(request)

        try:
            if 'function_id' in request.GET and 'version' in request.GET:
                self.prepare_source_fields_if_function_specified(request)
        except Exception:
            pass

    def prepare_source_fields_if_function_specified(self, request):
        try:
            function_id = request.GET['function_id']
            func = api.qinling.function_get(request, function_id)

            self.fields['func'].choices = [(function_id, func.name or func.id)]
            self.fields['func'].initial = function_id

            self.fields['version'].initial = request.GET['version']
        except Exception:
            msg = _('Unable to load the specified function. %s')
            exceptions.handle(request, msg % request.GET['function_id'])

    def clean(self):
        cleaned = super(CreateExecutionForm, self).clean()
        version_number = cleaned.get('version')

        # version number is not specified or specified as zero.
        if not version_number:
            return cleaned

        function_id = cleaned.get('func')

        versions = api.qinling.versions_list(self.request, function_id)

        # in case versions are returned as empty array.
        if not versions and not version_number:
            return cleaned

        available_versions = [v.version_number for v in versions]
        if version_number not in available_versions:
            msg = _('This function does not '
                    'have specified version number: %s') % version_number
            raise forms.ValidationError(msg)

        return cleaned

    def get_func_choices(self, request):
        try:
            functions = api.qinling.functions_list(request)
        except Exception:
            functions = []

        function_choices = [(f.id, f.name or f.id) for f in functions]
        if len(function_choices) == 0:
            function_choices = [('', _('No functions available.'))]
        return function_choices

    def handle(self, request, data):
        try:
            function_id = data['func']
            version = int(data['version'])
            sync = data['sync']
            inp = \
                q_utils.convert_raw_input_to_api_format(data['input_params'])
            api.qinling.execution_create(request, function_id, version,
                                         sync, inp)
            message = _('Created execution of "%s"') % function_id
            messages.success(request, message)
            return True
        except Exception as e:
            redirect = reverse("horizon:project:executions:index")
            exceptions.handle(request,
                              _("Unable to create execution. %s") % e,
                              redirect=redirect)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import unittest

from django.core.exceptions import ValidationError

from qinling_dashboard import validators


def provider_validate_key_value_pairs():
    provider = list()

    # blank check
    empty_row = [
        # blank string
        {'d': u'', 'raise': False},
        # multiple lines consist of blank string
        {'d': u'\n\n\n\n\n\n\n\n\n\n\n', 'raise': False},
        # multiple lines consist of blank string + valid row
        {'d': u'\n\n\n\n\n\n\n\n\n\n\nkey=value', 'raise': False},
        # multiple lines consist of blank string + valid row
        #  + multiple lines consist of blank string
        {'d': u'\n\n\n\n\n\n\n\n\n\n\nkey=value\n\n', 'raise': False},
    ]
    provider += empty_row

    # key part check

    # consist of valid character case
    key_check_normal = [
        # lower limit
        {'d': u'k=v', 'raise': False},
        # upper limit
        {'d': u'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ012345'
              u'6789!"#$%&\'()*+,-./:;<F>?@[\]^_`{|}~abcdefghijklmnopqrstu'
              u'vwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789!"#$%&\'()*+,-./:;'
              u'<F>?@[\]^_`{|}~abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQR'
              u'STUVWXYZ0123456789!"#$%=v', 'raise': False},
        # upper limit +1
        {'d': u'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ01234567'
              u'89!"#$%&\'()*+,-./:;<F>?@[\]^_`{|}~abcdefghijklmnopqrstuvwxy'
              u'zABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789!"#$%&\'()*+,-./:;<F>?@'
              u'[\]^_`{|}~abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWX'
              u'YZ0123456789!"#$%A=v', 'raise': True},
    ]
    provider += key_check_normal

    # key has initial blank string
    key_check_starts_space = [
        {'d': u' =v', 'raise': True},
        # upper limit
        {'d': u' abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ01234567'
              u'89!"#$%&\'()*+,-./:;<F>?@[\]^_`{|}~abcdefghijklmnopqrstuvwxyzA'
              u'BCDEFGHIJKLMNOPQRSTUVWXYZ0123456789!"#$%&\'()*+,-./:;<F>?@[\]^'
              u'_`{|}~abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123'
              u'456789!"#$=v', 'raise': True},
        # upper limit + 1
        {'d': u' abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ012345678'
              u'9!"#$%&\'()*+,-./:;<F>?@[\]^_`{|}~abcdefghijklmnopqrstuvwxyzAB'
              u'CDEFGHIJKLMNOPQRSTUVWXYZ0123456789!"#$%&\'()*+,-./:;<F>?@[\]^_'
              u'`{|}~abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ01234'
              u'56789!"#$A=v', 'raise': True},
    ]
    provider += key_check_starts_space

    # key has last blank string
    key_check_normal = [
        # upper limit
        {'d': u'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ012345678'
              u'9!"#$%&\'()*+,-./:;<F>?@[\]^_`{|}~abcdefghijklmnopqrstuvwxyzA'
              u'BCDEFGHIJKLMNOPQRSTUVWXYZ0123456789!"#$%&\'()*+,-./:;<F>?@[\]'
              u'^_`{|}~abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ01'
              u'23456789!"#$ =v', 'raise': True},
        # upper limit + 1
        {'d': u'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ012345678'
              u'9!"#$%&\'()*+,-./:;<F>?@[\]^_`{|}~abcdefghijklmnopqrstuvwxyzA'
              u'BCDEFGHIJKLMNOPQRSTUVWXYZ0123456789!"#$%&\'()*+,-./:;<F>?@[\]'
              u'^_`{|}~abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ01'
              u'23456789!"#$% =v', 'raise': True},
    ]
    provider += key_check_normal

    # key has blank string in the middle of it
    key_check_middle_space = [
        # upper limit
        {'d': u'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ012345678'
              u'9 !"#$%&\'()*+,-./:;<F>?@[\]^_`{|}~abcdefghijklmnopqrstuvwxyz'
              u'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789!"#$%&\'()*+,-./:;<F>?@[\]'
              u'^_`{|}~abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ012'
              u'3456789!"#$=v', 'raise': False},
        # upper limit + 1
        {'d': u'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'
              u' !"#$%&\'()*+,-./:;<F>?@[\]^_`{|}~abcdefghijklmnopqrstuvwxyzAB'
              u'CDEFGHIJKLMNOPQRSTUVWXYZ0123456789!"#$%&\'()*+,-./:;<F>?@[\]^_'
              u'`{|}~abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ01234'
              u'56789!"#$A=v', 'raise': True},
    ]
    provider += key_check_middle_space

    # check for key part
    no_key = [
        {'d': u'=v', 'raise': True},
    ]
    provider += no_key

    # equal part check
    equal_check = [
        # no equal
        {'d': 'key1value1', 'raise': True},
        # multiple equal in it
        {'d': 'key=value=key\n', 'raise': True},
    ]
    provider += equal_check

    # check for value part

    # consist of valid charcters
    value_check_normal = [
        # upper limit
        {'d': u'k=abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ01234567'
              u'89!"#$%&\'()*+,-./:;<F>?@[\]^_`{|}~abcdefghijklmnopqrstuvwxyzA'
              u'BCDEFGHIJKLMNOPQRSTUVWXYZ0123456789!"#$%&\'()*+,-./:;<F>?@[\]^'
              u'_`{|}~abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123'
              u'456789!"#$%', 'raise': False},
        # upper limit + 1
        {'d': u'k=abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ01234567'
              u'89!"#$%&\'()*+,-./:;<F>?@[\]^_`{|}~abcdefghijklmnopqrstuvwxyzA'
              u'BCDEFGHIJKLMNOPQRSTUVWXYZ0123456789!"#$%&\'()*+,-./:;<F>?@[\]^'
              u'_`{|}~abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123'
              u'456789!"#$%A', 'raise': True},
    ]
    provider += value_check_normal

    # value has initial blank
    value_check_starts_space = [
        {'d': u'k= ', 'raise': True},
        # upper limit
        {'d': u'k= abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456'
              u'789!"#$%&\'()*+,-./:;<F>?@[\]^_`{|}~abcdefghijklmnopqrstuvwxyz'
              u'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789!"#$%&\'()*+,-./:;<F>?@[\]'
              u'^_`{|}~abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ012'
              u'3456789!"#$', 'raise': True},
        # upper limit + 1
        {'d': u'k= abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456'
              u'789!"#$%&\'()*+,-./:;<F>?@[\]^_`{|}~abcdefghijklmnopqrstuvwxyz'
              u'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789!"#$%&\'()*+,-./:;<F>?@[\]'
              u'^_`{|}~abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ012'
              u'3456789!"#$A', 'raise': True},
    ]
    provider += value_check_starts_space

    # value has last blank
    value_check_normal = [
        # upper limit
        {'d': u'k=abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456'
              u'789!"#$%&\'()*+,-./:;<F>?@[\]^_`{|}~abcdefghijklmnopqrstuvwxy'
              u'zABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789!"#$%&\'()*+,-./:;<F>?@'
              u'[\]^_`{|}~abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWX'
              u'YZ0123456789!"#$ ', 'raise': True},
        # upper limit + 1
        {'d': u'k=abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456'
              u'789!"#$%&\'()*+,-./:;<F>?@[\]^_`{|}~abcdefghijklmnopqrstuvwxy'
              u'zABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789!"#$%&\'()*+,-./:;<F>?@'
              u'[\]^_`{|}~abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWX'
              u'YZ0123456789!"#$% ', 'raise': True},
    ]
    provider += value_check_normal

    # value has middle blank
    value_check_middle_space = [
        # upper limit
        {'d': u'k=abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ012345'
              u'6789 !"#$%&\'()*+,-./:;<F>?@[\]^_`{|}~abcdefghijklmnopqrstuv'
              u'wxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789!"#$%&\'()*+,-./:;<F'
              u'>?@[\]^_`{|}~abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTU'
              u'VWXYZ0123456789!"#$', 'raise': False},
        # upper limit + 1
        {'d': u'k=abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ012345'
              u'6789 !"#$%&\'()*+,-./:;<F>?@[\]^_`{|}~abcdefghijklmnopqrstuv'
              u'wxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789!"#$%&\'()*+,-./:;<F'
              u'>?@[\]^_`{|}~abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTU'
              u'VWXYZ0123456789!"#$A', 'raise': True},
    ]
    provider += value_check_middle_space

    # no value is specified
    no_value = [
        {'d': u'k=', 'raise': True},
    ]
    provider += no_value

    return provider


def provider_validate_one_line_string():
    return [

        # Threshold check for number of charaters

        # lower limit - 1
        {'d': u'', 'raise': True},
        # lower limit
        {'d': u'a', 'raise': False},
        # upper limit
        {'d': u"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVwXYZ012345"
              u"6789!\"#$%&'()*+,-./:;<=>?@[\]^_`{|}~ abcdefghijklmnopqrst"
              u"uvwxyzABCDEFGHIJKLMNOPQRSTUVwXYZ0123456789!\"#$%&'()*+,-./:"
              u";<=>?@[\]^_`{|}~ abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOP"
              u"QRSTUVwXYZ0123456789!\"#",
         'raise': False},
        # upper limit + 1
        {'d': u"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVwXYZ012345"
              u"6789!\"#$%&'()*+,-./:;<=>?@[\]^_`{|}~ abcdefghijklmnopqrst"
              u"uvwxyzABCDEFGHIJKLMNOPQRSTUVwXYZ0123456789!\"#$%&'()*+,-./"
              u":;<=>?@[\]^_`{|}~ abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMN"
              u"OPQRSTUVwXYZ0123456789!\"#$",
         'raise': True},

        # initial character is blank

        # lower limit
        {'d': u' ', 'raise': True},
        # upper limit
        {'d': u" bcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVwXYZ01234567"
              u"89!\"#$%&'()*+,-./:;<=>?@[\]^_`{|}~ abcdefghijklmnopqrstuvwx"
              u"yzABCDEFGHIJKLMNOPQRSTUVwXYZ0123456789!\"#$%&'()*+,-./:;<=>?"
              u"@[\]^_`{|}~ abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUV"
              u"wXYZ0123456789!\"#",
         'raise': True},
        # upper limit + 1
        {'d': u" bcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVwXYZ0123456"
              u"789!\"#$%&'()*+,-./:;<=>?@[\]^_`{|}~ abcdefghijklmnopqrstuv"
              u"wxyzABCDEFGHIJKLMNOPQRSTUVwXYZ0123456789!\"#$%&'()*+,-./:;<"
              u"=>?@[\]^_`{|}~ abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQR"
              u"STUVwXYZ0123456789!\"#a",
         'raise': True},

        # last character is blank

        # upper limit
        {'d': u"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVwXYZ0123456"
              u"789!\"#$%&'()*+,-./:;<=>?@[\]^_`{|}~ abcdefghijklmnopqrstuv"
              u"wxyzABCDEFGHIJKLMNOPQRSTUVwXYZ0123456789!\"#$%&'()*+,-./:;<"
              u"=>?@[\]^_`{|}~ abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQR"
              u"STUVwXYZ0123456789!\" """,
         'raise': True},
        # upper limit + 1
        {'d': u"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVwXYZ0123456"
              u"789!\"#$%&'()*+,-./:;<=>?@[\]^_`{|}~ abcdefghijklmnopqrstuv"
              u"wxyzABCDEFGHIJKLMNOPQRSTUVwXYZ0123456789!\"#$%&'()*+,-./:;<"
              u"=>?@[\]^_`{|}~ abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQR"
              u"STUVwXYZ0123456789!\"# """,
         'raise': True},

    ]


class ValidatorsTests(unittest.TestCase):

    def test_validate_metadata(self, data=provider_validate_key_value_pairs()):
        for datum in data:

            d = datum.get('d')
            raise_expected = datum.get('raise')

            if raise_expected:
                self.assertRaises(ValidationError,
                                  validators.validate_key_value_pairs, d)
            else:
                self.assertIsNone(validators.validate_key_value_pairs(d))

    def test_validate_openstack_string(
            self, data=provider_validate_one_line_string()):
        for datum in data:

            d = datum.get('d')
            raise_expected = datum.get('raise')

            if raise_expected:
                self.assertRaises(ValidationError,
                                  validators.validate_one_line_string, d)
            else:
                self.assertIsNone(validators.validate_one_line_string(d))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from django.utils.translation import ugettext_lazy as _

from horizon import tabs


LOG = logging.getLogger(__name__)


class ExecutionOverviewTab(tabs.Tab):
    name = _("Overview")
    slug = "overview"
    template_name = "project/executions/_detail_overview.html"

    def get_context_data(self, request):
        execution = self.tab_group.kwargs["execution"]
        return {"execution": execution,
                "result": execution.result}


class ExecutionLogsTab(tabs.Tab):
    name = _("Execution Logs")
    slug = "execution_logs"
    template_name = "project/executions/_detail_logs.html"

    def get_context_data(self, request):
        execution_logs = self.tab_group.kwargs["execution_logs"]
        return {"execution_logs": execution_logs}


class ExecutionDetailTabs(tabs.TabGroup):
    slug = "execution_details"
    tabs = (ExecutionOverviewTab,
            ExecutionLogsTab)
    sticky = True
    show_single_tab = False



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

import six

from django.conf import settings

from horizon.utils.memoized import memoized

from openstack_dashboard.api import base

from openstack_dashboard.contrib.developer.profiler import api as profiler

from qinlingclient import client as qinling_client


@memoized
def qinlingclient(request, password=None):
    api_version = "1"
    insecure = getattr(settings, 'OPENSTACK_SSL_NO_VERIFY', False)
    cacert = getattr(settings, 'OPENSTACK_SSL_CACERT', None)
    endpoint = base.url_for(request, 'function-engine')
    kwargs = {
        'token': request.user.token.id,
        'insecure': insecure,
        'ca_file': cacert,
        'username': request.user.username,
        'password': password
    }
    client = qinling_client.Client(api_version, endpoint, **kwargs)
    return client


@profiler.trace
def runtimes_list(request):
    return qinlingclient(request).runtimes.list()


@profiler.trace
def runtime_get(request, runtime_id):
    return qinlingclient(request).runtimes.get(runtime_id)


def runtime_create(request, **params):
    resource = qinlingclient(request).runtimes.create(**params)
    return resource


def set_code(datum):
    if isinstance(datum.code, six.string_types):
        code_dict = json.loads(datum.code)
        setattr(datum, "code", code_dict)


@profiler.trace
def functions_list(request, with_version=False):
    functions = qinlingclient(request).functions.list()
    for f in functions:
        set_code(f)

    if with_version:
        for f in functions:
            function_id = f.id
            my_versions = \
                qinlingclient(request).function_versions.list(function_id)
            setattr(f, 'versions', my_versions)
    return functions


@profiler.trace
def function_get(request, function_id):
    function = qinlingclient(request).functions.get(function_id)
    set_code(function)
    return function


@profiler.trace
def function_create(request, **params):
    resource = qinlingclient(request).functions.create(**params)
    return resource


@profiler.trace
def function_update(request, function_id, **params):
    resource = qinlingclient(request).functions.update(function_id, **params)
    return resource


@profiler.trace
def function_delete(request, function_id):
    qinlingclient(request).functions.delete(function_id)


@profiler.trace
def function_download(request, function_id):
    function = qinlingclient(request).functions.get(function_id,
                                                    download=True)
    return function


def set_result(datum):
    if isinstance(datum.result, six.string_types):
        result_dict = json.loads(datum.result)
        setattr(datum, "result", result_dict)


@profiler.trace
def executions_list(request, function_id=None):
    executions = qinlingclient(request).function_executions.list()

    for e in executions:
        set_result(e)

    if function_id:
        executions = [e for e in executions
                      if e.function_id == function_id]
    return executions


@profiler.trace
def execution_get(request, execution_id):
    execution = qinlingclient(request).function_executions.get(execution_id)
    set_result(execution)
    return execution


@profiler.trace
def execution_create(request, function_id, version=0,
                     sync=True, input=None):
    execution = qinlingclient(request).function_executions.\
        create(function_id, version, sync, input)
    return execution


@profiler.trace
def execution_delete(request, execution_id):
    qinlingclient(request).function_executions.delete(execution_id)


@profiler.trace
def execution_log_get(request, execution_id):
    try:
        jr = qinlingclient(request).\
            function_executions.http_client.json_request
        resp, body = jr('/v1/executions/%s/log' % execution_id, 'GET')
        raw_logs = resp._content
    except Exception:
        raw_logs = ""
    return raw_logs


@profiler.trace
def versions_list(request, function_id):
    versions = qinlingclient(request).function_versions.list(function_id)
    return versions


@profiler.trace
def version_get(request, function_id, version_number):
    version = qinlingclient(request).function_versions.get(function_id,
                                                           version_number)
    return version


@profiler.trace
def version_create(request, function_id, description=""):
    version = qinlingclient(request).function_versions.create(function_id,
                                                              description)
    return version


@profiler.trace
def version_delete(request, function_id, version_number):
    qinlingclient(request).function_versions.delete(function_id,
                                                    version_number)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from django.urls import reverse

from django.utils.http import urlunquote

from qinling_dashboard import api
from qinling_dashboard.test import helpers as test

import mock


INDEX_TEMPLATE = 'horizon/common/_data_table_view.html'
INDEX_URL = reverse('horizon:project:runtimes:index')


class RuntimesTests(test.TestCase):

    @test.create_mocks({
        api.qinling: [
            'runtime_create',
        ]})
    def test_execution_create_with_maximum_params(self):
        data_runtime = self.runtimes.first()

        self.mock_runtime_create.return_value = data_runtime

        image_name = 'dummy/dockerimage'
        form_data = {'image': image_name,
                     'name': 'test_name',
                     'description': 'description',
                     'untrusted': 'on'}

        url = reverse('horizon:project:runtimes:create')
        res = self.client.post(url, form_data)

        self.assertNoFormErrors(res)
        self.assertRedirectsNoFollow(res, INDEX_URL)

        self.mock_runtime_create.assert_called_once_with(
            test.IsHttpRequest(),
            image=image_name,
            name='test_name',
            description='description',
            trusted=False)

    @test.create_mocks({
        api.qinling: [
            'runtime_create',
        ]})
    def test_execution_create_with_minimum_params(self):
        data_runtime = self.runtimes.first()

        self.mock_runtime_create.return_value = data_runtime

        image_name = 'dummy/dockerimage'
        form_data = {'image': image_name}

        url = reverse('horizon:project:runtimes:create')
        res = self.client.post(url, form_data)

        self.assertNoFormErrors(res)
        self.assertRedirectsNoFollow(res, INDEX_URL)

        self.mock_runtime_create.assert_called_once_with(
            test.IsHttpRequest(), image=image_name)

    @test.create_mocks({
        api.qinling: [
            'runtimes_list',
        ]})
    def test_index(self):
        data_runtimes = self.runtimes.list()
        self.mock_runtimes_list.return_value = data_runtimes

        res = self.client.get(INDEX_URL)

        self.assertTemplateUsed(res, INDEX_TEMPLATE)
        runtimes = res.context['runtimes_table'].data
        self.assertItemsEqual(runtimes, self.runtimes.list())

        self.mock_runtimes_list.assert_has_calls(
            [
                mock.call(test.IsHttpRequest()),
            ])

    @test.create_mocks({
        api.qinling: [
            'runtimes_list',
        ]})
    def test_index_runtimes_list_returns_exception(self):
        self.mock_runtimes_list.side_effect = self.exceptions.qinling

        res = self.client.get(INDEX_URL)

        self.assertTemplateUsed(res, INDEX_TEMPLATE)

        self.assertEqual(len(res.context['runtimes_table'].data), 0)
        self.assertMessageCount(res, error=1)

        self.mock_runtimes_list.assert_has_calls(
            [
                mock.call(test.IsHttpRequest()),
            ])

    @test.create_mocks({
        api.qinling: [
            'runtime_get',
        ]})
    def test_detail(self):
        runtime_id = self.runtimes.first().id
        self.mock_runtime_get.return_value = self.runtimes.first()

        url = urlunquote(reverse('horizon:project:runtimes:detail',
                                 args=[runtime_id]))
        res = self.client.get(url)

        result_runtime = res.context['runtime']

        self.assertEqual(runtime_id, result_runtime.id)

        self.assertTemplateUsed(res, 'project/runtimes/detail.html')

        self.mock_runtime_get.assert_has_calls(
            [
                mock.call(test.IsHttpRequest(), runtime_id),
            ])

    @test.create_mocks({
        api.qinling: [
            'runtime_get',
        ]})
    def test_detail_runtime_get_returns_exception(self):
        runtime_id = self.runtimes.first().id
        self.mock_runtime_get.side_effect = self.exceptions.qinling

        url = urlunquote(reverse('horizon:project:runtimes:detail',
                                 args=[runtime_id]))

        res = self.client.get(url)

        redir_url = INDEX_URL
        self.assertRedirectsNoFollow(res, redir_url)

        self.mock_runtime_get.assert_has_calls(
            [
                mock.call(test.IsHttpRequest(), runtime_id),
            ])



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from django.urls import reverse
from django.urls import reverse_lazy
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import tables
from horizon import tabs

from horizon.utils import memoized

from qinling_dashboard import api
from qinling_dashboard.content.executions import forms as project_forms
from qinling_dashboard.content.executions import tables as project_tables
from qinling_dashboard.content.executions import tabs as project_tabs


class CreateExecutionView(forms.ModalFormView):

    form_class = project_forms.CreateExecutionForm
    modal_header = submit_label = page_title = _("Create Execution")
    template_name = 'project/executions/create.html'
    submit_url = reverse_lazy("horizon:project:executions:create")
    success_url = reverse_lazy("horizon:project:executions:index")


class IndexView(tables.DataTableView):

    table_class = project_tables.ExecutionsTable
    page_title = _("Executions")

    def get_data(self):
        try:
            executions = api.qinling.executions_list(self.request)
        except Exception:
            executions = []
            msg = _('Unable to retrieve executions list.')
            exceptions.handle(self.request, msg)
        return executions


class DetailView(tabs.TabbedTableView):
    tab_group_class = project_tabs.ExecutionDetailTabs
    template_name = 'project/executions/detail.html'
    page_title = _("Execution Details: {{ resource_name }}")
    failure_url = reverse_lazy('horizon:project:executions:index')

    @memoized.memoized_method
    def _get_data(self):
        execution_id = self.kwargs['execution_id']
        try:
            execution = api.qinling.execution_get(self.request, execution_id)
        except Exception:
            exceptions.handle(
                self.request,
                _('Unable to retrieve '
                  'details for execution "%s".') % execution_id,
                redirect=self.failure_url
            )
        return execution

    def _get_execution_logs(self):
        execution_id = self.kwargs['execution_id']
        try:
            execution_logs = api.qinling.execution_log_get(self.request,
                                                           execution_id)
        except Exception:
            execution_logs = ""
            msg = _('Unable to retrieve execution logs.')
            exceptions.handle(self.request, msg)
        return execution_logs

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        execution = self._get_data()
        table = project_tables.ExecutionsTable(self.request)
        context["execution"] = execution
        context["url"] = self.get_redirect_url()
        context["actions"] = table.render_row_actions(execution)
        context["table_id"] = "execution"
        context["resource_name"] = execution.id
        context["object_id"] = execution.id
        return context

    def get_tabs(self, request, *args, **kwargs):
        execution = self._get_data()
        execution_logs = self._get_execution_logs()
        return self.tab_group_class(request,
                                    execution=execution,
                                    execution_logs=execution_logs,
                                    **kwargs)

    @staticmethod
    def get_redirect_url():
        return reverse('horizon:project:executions:index')



# Copyright 2012 Nebula, Inc.
# All rights reserved.

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from django.urls import reverse

from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import messages

from qinling_dashboard import api

from qinling_dashboard.utils import calculate_md5

from qinling_dashboard import validators


CPU_MIN_VALUE = 100
CPU_MAX_VALUE = 300
MEMORY_MIN_VALUE = 33554432
MEMORY_MAX_VALUE = 134217728


CPU_HELP_TEXT = _("Limit of cpu resource(unit: millicpu). Range: {0} ~ {1}").\
    format(CPU_MIN_VALUE, CPU_MAX_VALUE)

MEMORY_HELP_TEXT = _("Limit of memory resource(unit: bytes). "
                     "Range: {0} ~ {1} (bytes).").format(MEMORY_MIN_VALUE,
                                                         MEMORY_MAX_VALUE)

UPLOAD_PACKAGE_LABEL = _('Package')
UPLOAD_SWIFT_CONTAINER_LABEL = _('Swift Container')
UPLOAD_SWIFT_OBJECT_LABEL = _('Swift Object')
UPLOAD_IMAGE_LABEL = _('Image')

CODE_TYPE_CHOICES = [('package', UPLOAD_PACKAGE_LABEL),
                     ('swift', UPLOAD_SWIFT_OBJECT_LABEL),
                     ('image', UPLOAD_IMAGE_LABEL)]


class CreateFunctionForm(forms.SelfHandlingForm):

    name = forms.CharField(max_length=255,
                           label=_("Name"),
                           validators=[validators.validate_one_line_string],
                           required=False)

    description = forms.CharField(
        max_length=255,
        widget=forms.Textarea(
            attrs={'class': 'modal-body-fixed-width',
                   'rows': 3}),
        label=_("Description"),
        required=False)

    cpu = forms.IntegerField(label=_("CPU"),
                             help_text=CPU_HELP_TEXT,
                             min_value=CPU_MIN_VALUE,
                             max_value=CPU_MAX_VALUE,
                             required=False)

    memory_size = forms.IntegerField(label=_("Memory Size"),
                                     help_text=MEMORY_HELP_TEXT,
                                     min_value=MEMORY_MIN_VALUE,
                                     max_value=MEMORY_MAX_VALUE,
                                     required=False)

    code_type = forms.ThemableChoiceField(
        label=_("Code Type"),
        help_text=_("Select Your Code Type."),
        widget=forms.ThemableSelectWidget(
            attrs={'class': 'switchable',
                   'data-slug': 'code_type'}
        ),
        choices=CODE_TYPE_CHOICES,
        required=True)

    package_file = forms.FileField(
        label=UPLOAD_PACKAGE_LABEL,
        help_text=_('Code package zip file path.'),
        widget=forms.FileInput(
            attrs={
                'class': 'switched',
                'data-switch-on': 'code_type',
                'data-code_type-package': UPLOAD_PACKAGE_LABEL,
            },
        ),
        required=False)

    entry = forms.CharField(
        label=_("Entry"),
        help_text=_('Function entry in the format of '
                    '<module_name>.<method_name>'),
        validators=[validators.validate_one_line_string],
        widget=forms.TextInput(attrs={
            'class': 'switched',
            'data-switch-on': 'code_type',
            'data-code_type-package': _("Entry")
        }),
        required=False)

    runtime = forms.ThemableChoiceField(
        label=_("Runtime"),
        widget=forms.SelectWidget(attrs={
            'class': 'switched',
            'data-switch-on': 'code_type',
            'data-code_type-package': _("Runtime")
        }),
        required=False)

    swift_container = forms.CharField(
        label=UPLOAD_SWIFT_CONTAINER_LABEL,
        help_text=_('Container name in Swift.'),
        validators=[validators.validate_one_line_string],
        widget=forms.TextInput(attrs={
            'class': 'switched',
            'data-switch-on': 'code_type',
            'data-code_type-swift': UPLOAD_SWIFT_CONTAINER_LABEL
        }),
        required=False)

    swift_object = forms.CharField(
        label=UPLOAD_SWIFT_OBJECT_LABEL,
        help_text=_('Object name in Swift.'),
        validators=[validators.validate_one_line_string],
        widget=forms.TextInput(attrs={
            'class': 'switched',
            'data-switch-on': 'code_type',
            'data-code_type-swift': UPLOAD_SWIFT_OBJECT_LABEL
        }),
        required=False)

    entry_swift = forms.CharField(
        label=_("Entry"),
        help_text=_('Function entry in the format of '
                    '<module_name>.<method_name>'),
        validators=[validators.validate_one_line_string],
        widget=forms.TextInput(attrs={
            'class': 'switched',
            'data-switch-on': 'code_type',
            'data-code_type-swift': _("Entry")
        }),
        required=False)

    runtime_swift = forms.ThemableChoiceField(
        label=_("Runtime"),
        widget=forms.SelectWidget(attrs={
            'class': 'switched',
            'data-switch-on': 'code_type',
            'data-code_type-swift': _("Runtime")
        }),
        required=False)

    image = forms.CharField(
        label=UPLOAD_IMAGE_LABEL,
        help_text=_('Image name in Docker hub.'),
        validators=[validators.validate_one_line_string],
        widget=forms.TextInput(attrs={
            'class': 'switched',
            'data-switch-on': 'code_type',
            'data-code_type-image': UPLOAD_IMAGE_LABEL
        }),
        required=False)

    def get_runtime_choices(self, request):
        try:
            runtimes = api.qinling.runtimes_list(request)
        except Exception:
            runtimes = []
            exceptions.handle(request, _('Unable to retrieve runtimes.'))

        runtime_list = [(runtime.id, runtime.name) for runtime in runtimes
                        if runtime.status == 'available']

        runtime_list.sort()

        if not runtime_list:
            runtime_list.insert(0, ("", _("No runtimes found")))
        return runtime_list

    def __init__(self, request, *args, **kwargs):
        super(CreateFunctionForm, self).__init__(request, *args, **kwargs)
        runtime_choices = self.get_runtime_choices(request)
        self.fields['runtime'].choices = runtime_choices
        self.fields['runtime_swift'].choices = runtime_choices

    def _validation_for_swift_case(self, cleaned):
        swift_container = cleaned.get('swift_container', None)
        swift_object = cleaned.get('swift_object', None)
        if not all([swift_container, swift_object]):
            msg = _('You must specify container and object '
                    'both in case code type is Swift.')
            raise forms.ValidationError(msg)

        runtime = cleaned.get('runtime_swift', None)
        if not runtime:
            msg = _('You must specify runtime.')
            raise forms.ValidationError(msg)

    def _validation_for_image_case(self, cleaned):
        image = cleaned.get('image', None)
        if not image:
            msg = _('You must specify Docker image.')
            raise forms.ValidationError(msg)

    def _get_package_file(self, cleaned):
        package_file = cleaned.get('package_file', None)
        return package_file

    def _validation_for_package_case(self, cleaned):
        package_file = self._get_package_file(cleaned)

        if not package_file:
            msg = _('You must specify package file.')
            raise forms.ValidationError(msg)

        runtime = cleaned.get('runtime', None)
        if not runtime:
            msg = _('You must specify runtime.')
            raise forms.ValidationError(msg)

    def clean(self):
        cleaned = super(CreateFunctionForm, self).clean()

        code_type = cleaned['code_type']

        if code_type == 'package':
            self._validation_for_package_case(cleaned)

        if code_type == 'swift':
            self._validation_for_swift_case(cleaned)

        if code_type == 'image':
            self._validation_for_image_case(cleaned)

        return cleaned

    def handle_package_case(self, params, context, update=False):
        upload_files = self.request.FILES

        package = upload_files.get('package_file')
        md5sum = calculate_md5(package)
        code = {'source': 'package', 'md5sum': md5sum}

        # case1: creation case.
        #     Package upload is required in creation case.
        # case2: update case.
        #     In case update, package/code are only added
        #     when user specify them.
        if not update or (update and package):
            package = [ck for ck in package.chunks()][0]
            params.update({
                'package': package,
                'code': code,
            })

        if not update and context.get('runtime'):
            params.update({'runtime': context.get('runtime')})

        self._handle_entry_for_package(params, context, update)

    def _handle_entry_for_package(self, params, context, update=False):
        if update:
            params.update({'entry': context.get('entry')})
        else:
            if context.get('entry'):
                params.update({'entry': context.get('entry')})

    def handle_swift_case(self, params, context, update=False):
        swift_container = context.get('swift_container')
        swift_object = context.get('swift_object')

        if not update or all([not update, swift_container, swift_object]):
            code = {
                'source': 'swift',
                'swift': {
                    'container': swift_container,
                    'object': swift_object
                }
            }
            params.update({'code': code})

        if not update and context.get('runtime_swift'):
            params.update({'runtime': context.get('runtime_swift')})

        if not update and context.get('entry_swift'):
            params.update({'entry': context.get('entry_swift')})

    def handle_image_case(self, params, context, update=False):
        if not update or all([not update, context.get('image')]):
            code = {
                'source': 'image',
                'image': context.get('image')
            }
            params.update({'code': code})

    def handle(self, request, context):
        params = {}

        # basic parameters
        if context.get('name'):
            params.update({'name': context.get('name')})

        if context.get('description'):
            params.update({'description': context.get('description')})

        if context.get('cpu'):
            params.update({'cpu': int(context.get('cpu'))})

        if context.get('memory_size'):
            params.update({'memory_size': int(context.get('memory_size'))})

        code_type = context.get('code_type')

        if code_type == 'package':
            self.handle_package_case(params, context)

        elif code_type == 'swift':
            self.handle_swift_case(params, context)

        elif code_type == 'image':
            self.handle_image_case(params, context)

        try:
            api.qinling.function_create(request, **params)
            message = _('Created function "%s"') % params.get('name',
                                                              'unknown name')
            messages.success(request, message)
            return True
        except Exception:
            redirect = reverse("horizon:project:functions:index")
            exceptions.handle(request,
                              _("Unable to create function."),
                              redirect=redirect)


class UpdateFunctionForm(CreateFunctionForm):

    function_id = forms.CharField(widget=forms.HiddenInput(),
                                  required=False)

    code_type = forms.ThemableChoiceField(
        label=_("Code Type"),
        help_text=_("Code Type can not be changed when you update function."),
        widget=forms.TextInput(
            attrs={'readonly': 'readonly'}
        ),
        choices=CODE_TYPE_CHOICES,
        required=True)

    # override this to skip
    def clean(self):
        cleaned = super(forms.SelfHandlingForm, self).clean()
        return cleaned

    def __init__(self, request, *args, **kwargs):
        super(UpdateFunctionForm, self).__init__(request, *args, **kwargs)

        code_type = self.initial.get('code_type')

        # written field names here will be regarded as updatable
        common_fields = ['name', 'description',
                         'code_type', 'function_id']
        available_fields = {
            'package': ['package_file', 'entry', 'cpu', 'memory_size'],
            'swift': [],
            'image': [],
        }
        available_fields = available_fields[code_type] + common_fields
        field_names = [fn for fn in self.fields.keys()]
        for field_name in field_names:
            if field_name not in available_fields:
                del self.fields[field_name]

    def handle(self, request, context):
        # basic parameters
        params = {
            'name': context.get('name', ''),
            'description': context.get('description', '')
        }

        if context.get('cpu'):
            params.update({'cpu': int(context.get('cpu'))})

        if context.get('memory_size'):
            params.update({'memory_size': int(context.get('memory_size'))})

        code_type = context.get('code_type')

        if code_type == 'package':
            self.handle_package_case(params, context, update=True)

        elif code_type == 'swift':
            self.handle_swift_case(params, context, update=True)

        elif code_type == 'image':
            self.handle_image_case(params, context, update=True)

        function_id = context['function_id']
        try:
            api.qinling.function_update(request, function_id, **params)
            message = _('Updated function "%s"') % params.get('name',
                                                              'unknown name')
            messages.success(request, message)
            return True
        except Exception:
            redirect = reverse("horizon:project:functions:index")
            exceptions.handle(request,
                              _("Unable to update function %s") % function_id,
                              redirect=redirect)


class CreateFunctionVersionForm(forms.SelfHandlingForm):

    function_id = forms.CharField(
        label=_("Function ID"),
        help_text=_("Function which new version will be created."),
        widget=forms.TextInput(
            attrs={'readonly': 'readonly'}
        ),
        required=False)

    description = forms.CharField(
        max_length=255,
        widget=forms.Textarea(
            attrs={'class': 'modal-body-fixed-width',
                   'rows': 3}),
        label=_("Description"),
        required=False)

    def handle(self, request, data):
        try:
            function_id = data['function_id']
            description = data['description']
            api.qinling.version_create(request, function_id, description)
            message = _('Created new version of "%s"') % function_id
            messages.success(request, message)
            return True
        except Exception as e:
            redirect = reverse("horizon:project:functions:index")
            if hasattr(e, 'details'):
                msg = _("Unable to create execution. %s") % e.details
            else:
                msg = _("Unable to create execution")
            exceptions.handle(request, msg, redirect=redirect)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from django.urls import reverse
from django.urls import reverse_lazy
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import tables
from horizon import tabs
from horizon.utils import memoized

from qinling_dashboard import api
from qinling_dashboard.content.runtimes import forms as project_forms
from qinling_dashboard.content.runtimes import tables as project_tables
from qinling_dashboard.content.runtimes import tabs as project_tabs


class CreateRuntimeView(forms.ModalFormView):

    form_class = project_forms.CreateRuntimeForm
    modal_header = submit_label = page_title = _("Create Runtime")
    template_name = 'project/runtimes/create.html'
    submit_url = reverse_lazy("horizon:project:runtimes:create")
    success_url = reverse_lazy("horizon:project:runtimes:index")


class IndexView(tables.DataTableView):

    table_class = project_tables.RuntimesTable
    page_title = _("Runtimes")

    def get_data(self):
        try:
            runtimes = api.qinling.runtimes_list(self.request)
        except Exception:
            runtimes = []
            msg = _('Unable to retrieve runtimes list.')
            exceptions.handle(self.request, msg)
        return runtimes


class DetailView(tabs.TabView):
    tab_group_class = project_tabs.RuntimeDetailTabs
    template_name = 'project/runtimes/detail.html'
    page_title = _("Runtime Details: {{ resource_name }}")
    failure_url = reverse_lazy('horizon:project:runtimes:index')

    @memoized.memoized_method
    def get_data(self):
        runtime_id = self.kwargs['runtime_id']
        try:
            runtime = api.qinling.runtime_get(self.request, runtime_id)
        except Exception:
            exceptions.handle(
                self.request,
                _('Unable to retrieve '
                  'details for Runtime "%s".') % runtime_id,
                redirect=self.failure_url
            )
        return runtime

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        runtime = self.get_data()
        table = project_tables.RuntimesTable(self.request)
        resource_name = runtime.name if runtime.name else runtime.id
        context["runtime"] = runtime
        context["url"] = self.get_redirect_url()
        context["actions"] = table.render_row_actions(runtime)
        context["table_id"] = "runtime"
        context["resource_name"] = resource_name
        context["object_id"] = runtime.id
        return context

    def get_tabs(self, request, *args, **kwargs):
        runtime = self.get_data()
        return self.tab_group_class(request,
                                    runtime=runtime,
                                    **kwargs)

    @staticmethod
    def get_redirect_url():
        return reverse('horizon:project:runtimes:index')
